{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Author: *\n",
    "\n",
    "Date: 27.11.2017"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "import netgen.gui\n",
    "from netgen.geom2d import *\n",
    "from ngsolve import *\n",
    "from ngsolve.internal import *\n",
    "import numpy as np\n",
    "import time\n",
    "\n",
    "%gui tk"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "class MyGeometry:\n",
    "    def __init__(self, diameter, alpha, verticalDistance, rows, columns, canalWidth, canalLength, radiusI, sourceHeight):\n",
    "        self.diameter = diameter\n",
    "        self.alpha = alpha\n",
    "        self.verticalDistance = verticalDistance\n",
    "        self.horizontalDistance = 2*verticalDistance*sin(alpha/2)\n",
    "        self.rows = rows\n",
    "        self.columns = columns\n",
    "        self.canalWidth = canalWidth\n",
    "        self.canalLength = canalLength\n",
    "        self.radiusI = radiusI\n",
    "        self.geo = SplineGeometry()\n",
    "        self.xMid=self.columns*self.horizontalDistance/2.0\n",
    "        self.sourceHeight = sourceHeight\n",
    "        self.createRest()\n",
    "     \n",
    "    def createMesh(self):                    \n",
    "        self.geo.SetMaterial (1, \"matrix\")\n",
    "        self.geo.SetMaterial (2, \"hole\")\n",
    "        self.geo.SetMaterial (3, \"entry\")\n",
    "        self.geo.SetMaterial (4, \"air\")\n",
    "        self.geo.SetMaterial (5, \"plm\")\n",
    "        self.geo.SetMaterial (6, \"source\")\n",
    "        self.mesh = Mesh(self.geo.GenerateMesh(maxh=0.08))\n",
    "        \n",
    "    def createRest(self):\n",
    "        xMid=self.columns*self.horizontalDistance/2.0\n",
    "        width=self.columns*self.horizontalDistance\n",
    "        height=self.rows*self.verticalDistance\n",
    "        p1,p2,p3,p4,p5,p6=[self.geo.AppendPoint(x,y) for x,y in [(0,0), (xMid-self.canalWidth/2,0),(xMid+self.canalWidth/2,0), (width,0), (width,height), (0,height)]]\n",
    "        self.geo.Append ([\"line\", p1, p2], leftdomain=1, rightdomain=0)\n",
    "        self.geo.Append ([\"line\", p2, p3], leftdomain=1, rightdomain=3)\n",
    "        self.geo.Append ([\"line\", p3, p4], leftdomain=1, rightdomain=0)\n",
    "        self.geo.Append ([\"line\", p4, p5], leftdomain=1, rightdomain=4)       \n",
    "        self.geo.Append ([\"line\", p5, p6], leftdomain=1, rightdomain=4)\n",
    "        self.geo.Append ([\"line\", p6, p1], leftdomain=1, rightdomain=4)          \n",
    "        \n",
    "        p7,p8=[self.geo.AppendPoint(x,y) for x,y in [(xMid-self.canalWidth/2,-self.canalLength),(xMid+self.canalWidth/2,-self.canalLength)]]       \n",
    "        self.geo.Append ([\"line\", p2, p7], leftdomain=3, rightdomain=0)       \n",
    "        self.geo.Append ([\"line\", p7, p8], leftdomain=3, rightdomain=6)\n",
    "        self.geo.Append ([\"line\", p8, p3], leftdomain=3, rightdomain=0)\n",
    "        \n",
    "        p19,p20=[self.geo.AppendPoint(x,y) for x,y in [(xMid-self.canalWidth/2,-self.canalLength-self.sourceHeight),(xMid+self.canalWidth/2,-self.canalLength-self.sourceHeight)]]       \n",
    "        self.geo.Append ([\"line\", p7, p19], leftdomain=6, rightdomain=0)       \n",
    "        self.geo.Append ([\"line\", p19, p20], leftdomain=6, rightdomain=0)\n",
    "        self.geo.Append ([\"line\", p20, p8], leftdomain=6, rightdomain=0)        \n",
    " \n",
    "        p9,p10,p11,p12,p13=[self.geo.AppendPoint(x,y) for x,y in [(xMid+self.radiusI,0),(xMid+self.radiusI,self.radiusI),(xMid,self.radiusI),(xMid-self.radiusI,self.radiusI),(xMid-self.radiusI,0)]]       \n",
    "        self.geo.Append ([\"line\", p4, p9], leftdomain=4, rightdomain=0)       \n",
    "        self.geo.Append ([\"spline3\", p9, p10, p11], leftdomain=4, rightdomain=5)\n",
    "        self.geo.Append ([\"spline3\", p11, p12, p13], leftdomain=4, rightdomain=5)        \n",
    "        self.geo.Append ([\"line\", p13, p1], leftdomain=4, rightdomain=0)\n",
    "        \n",
    "        radiusO=self.radiusI*1.2\n",
    "        p14,p15,p16,p17,p18=[self.geo.AppendPoint(x,y) for x,y in [(xMid+radiusO,0),(xMid+radiusO,radiusO),(xMid,radiusO),(xMid-radiusO,radiusO),(xMid-radiusO,0)]]       \n",
    "        self.geo.Append ([\"line\", p9, p14], leftdomain=5, rightdomain=0)       \n",
    "        self.geo.Append ([\"spline3\", p14, p15, p16], leftdomain=5, rightdomain=0)\n",
    "        self.geo.Append ([\"spline3\", p16, p17, p18], leftdomain=5, rightdomain=0)        \n",
    "        self.geo.Append ([\"line\", p18, p13], leftdomain=5, rightdomain=0)\n",
    "        \n",
    "        \n",
    "    def createWithoutHoles(self):\n",
    "        self.createMesh()\n",
    "        \n",
    "    def createEquidistantGeometry(self):\n",
    "        for r in range(self.rows):\n",
    "            for c in range(self.columns):\n",
    "                if(r % 2 ==0):\n",
    "                    xValue = self.horizontalDistance/2+c*self.horizontalDistance\n",
    "                    yValue = self.verticalDistance/2+r*self.verticalDistance\n",
    "                    self.geo.AddCircle(c=(xValue,yValue),r=self.diameter/2,bc=\"circle\",leftdomain=2,rightdomain=1)\n",
    "                elif(c!=self.columns-1):\n",
    "                    xValue = self.horizontalDistance+c*self.horizontalDistance\n",
    "                    yValue = self.verticalDistance/2+r*self.verticalDistance\n",
    "                    self.geo.AddCircle(c=(xValue,yValue),r=self.diameter/2,bc=\"circle\",leftdomain=2,rightdomain=1)\n",
    "        self.createMesh()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "myGeo = MyGeometry(0.02,90,0.02,37,37,0.2,0.5,1.1,0.04)\n",
    "myGeo.createEquidistantGeometry()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [
    {
     "ename": "TypeError",
     "evalue": "SetElementOrder(): incompatible function arguments. The following argument types are supported:\n    1. (self: ngsolve.comp.Mesh, arg0: ngsolve.comp.ElementId, arg1: int) -> None\n\nInvoked with: <ngsolve.comp.Mesh object at 0x000001EE83190B48>, 5",
     "output_type": "error",
     "traceback": [
      "\u001b[1;31m---------------------------------------------------------------------------\u001b[0m",
      "\u001b[1;31mTypeError\u001b[0m                                 Traceback (most recent call last)",
      "\u001b[1;32m<ipython-input-4-0c3656429ee4>\u001b[0m in \u001b[0;36m<module>\u001b[1;34m()\u001b[0m\n\u001b[0;32m      3\u001b[0m \u001b[1;31m#Question:\u001b[0m\u001b[1;33m\u001b[0m\u001b[1;33m\u001b[0m\u001b[0m\n\u001b[0;32m      4\u001b[0m \u001b[1;31m#How to set the ElementOrder to 5?\u001b[0m\u001b[1;33m\u001b[0m\u001b[1;33m\u001b[0m\u001b[0m\n\u001b[1;32m----> 5\u001b[1;33m \u001b[0mmesh\u001b[0m\u001b[1;33m.\u001b[0m\u001b[0mSetElementOrder\u001b[0m\u001b[1;33m(\u001b[0m\u001b[1;36m5\u001b[0m\u001b[1;33m)\u001b[0m\u001b[1;33m\u001b[0m\u001b[0m\n\u001b[0m\u001b[0;32m      6\u001b[0m \u001b[1;31m#\u001b[0m\u001b[1;33m\u001b[0m\u001b[1;33m\u001b[0m\u001b[0m\n\u001b[0;32m      7\u001b[0m \u001b[0mmesh\u001b[0m\u001b[1;33m.\u001b[0m\u001b[0mSetPML\u001b[0m\u001b[1;33m(\u001b[0m\u001b[0mpml\u001b[0m\u001b[1;33m.\u001b[0m\u001b[0mRadial\u001b[0m\u001b[1;33m(\u001b[0m\u001b[0mrad\u001b[0m\u001b[1;33m=\u001b[0m\u001b[0mmyGeo\u001b[0m\u001b[1;33m.\u001b[0m\u001b[0mradiusI\u001b[0m\u001b[1;33m,\u001b[0m \u001b[0malpha\u001b[0m\u001b[1;33m=\u001b[0m\u001b[1;36m1j\u001b[0m\u001b[1;33m,\u001b[0m \u001b[0morigin\u001b[0m\u001b[1;33m=\u001b[0m\u001b[1;33m(\u001b[0m\u001b[0mmyGeo\u001b[0m\u001b[1;33m.\u001b[0m\u001b[0mxMid\u001b[0m\u001b[1;33m,\u001b[0m \u001b[1;36m0\u001b[0m\u001b[1;33m)\u001b[0m\u001b[1;33m)\u001b[0m\u001b[1;33m,\u001b[0m \u001b[1;34m\"pml\"\u001b[0m\u001b[1;33m)\u001b[0m\u001b[1;33m\u001b[0m\u001b[0m\n",
      "\u001b[1;31mTypeError\u001b[0m: SetElementOrder(): incompatible function arguments. The following argument types are supported:\n    1. (self: ngsolve.comp.Mesh, arg0: ngsolve.comp.ElementId, arg1: int) -> None\n\nInvoked with: <ngsolve.comp.Mesh object at 0x000001EE83190B48>, 5"
     ]
    }
   ],
   "source": [
    "mesh=myGeo.mesh\n",
    "\n",
    "#Question:\n",
    "#How to set the ElementOrder to 5?\n",
    "mesh.SetElementOrder(5)\n",
    "#\n",
    "mesh.SetPML(pml.Radial(rad=myGeo.radiusI, alpha=1j, origin=(myGeo.xMid, 0)), \"pml\")\n",
    "viewoptions.drawoutline=1\n",
    "Draw(mesh)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "fes = H1(mesh, order = 5, complex=True)\n",
    "u = fes.TrialFunction()\n",
    "v = fes.TestFunction()\n",
    "\n",
    "gfu = GridFunction(fes)\n",
    "fes.ndof"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.3"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
